/*
 * NSD (c) 2003 Netsweeper Inc. All Rights Reserved.
 * This application is designed to work with the Netsweeper Internet Content Filtering System.
 *
 * NSD is a trademark of Netsweeper Inc.
 * Netsweeper, Netsweeper Inc. is either a copyrighted or registered trademark of Netsweeper Inc.
 *
 * Programmers:
 *	Inital		Who
 *	----------------------------------------------
 *	JDE		Jeremy Erb
 *
 * History:
 *	When		Who	What
 *	----------------------------------------------
 *	2004-11-05	JDE	Implemented a nice IP List
 *
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <strings.h>
#include <unistd.h>
#endif
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#ifdef WIN32
#define nslog_send
#define NSLOG_MESSAGE 0
#define current_time (long)timeGetTime()
#include <winsock2.h>
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/poll.h>
#include <netdb.h>
#include <pthread.h>
#endif
#include <sys/types.h>
#include <signal.h>

#include "simple_iplist.h"

static int simple_iplist_ipv4_add_string(struct simple_iphead* head, char * ipaddr, void *data);
static int simple_iplist_ipv6_add_string(struct simple_iphead* head, char * ipaddr, void *data);
static void simple_iplist_ipv6_mask_cidr(struct in6_addr *ip, int cidr);
// Characters in IP ranges
#define OCTET_SEPARATOR '.'
#define RANGE_SEPARATOR '-'

//returns true if str is an ip address -- 4 numbers, dot sep
static int simple_iplist_ipaddr_check(const char *str)
{
	int i=0;
	int state=1;

	while((state>0)&&(str[i])) {
		switch(state) {
			case 1:
				if(isdigit((int)str[i]))
					state=2;
				else
					state=-1;
				break;
			case 2:
				if(str[i]=='.')
					state=3;
				else if(!isdigit((int)str[i]))
					state=-1;
				break;
			case 3:
				if(isdigit((int)str[i]))
					state=4;
				else
					state=-1;
				break;
			case 4:
				if(str[i]=='.')
					state=5;
				else if(!isdigit((int)str[i]))
					state=-1;
				break;
			case 5:
				if(isdigit((int)str[i]))
					state=6;
				else
					state=-1;
				break;
			case 6:
				if(str[i]=='.')
					state=7;
				else if(!isdigit((int)str[i]))
					state=-1;
				break;
			case 7:
				if(isdigit((int)str[i]))
					state=8;
				else
					state=-1;
				break;
			case 8: /* accept */
				if(str[i] =='/')
					state = 9;
				else if(!isdigit((int)str[i]))
					state=-1;
				break;
			case 9:
				if(!isdigit((int)str[i]))
					state=-1;
				break;
		}
		i++;
	}

	return((state==8)||(state==9));
}

int simple_iplist_add_string(struct simple_iphead* head, char * ipaddr, void *data)
{
	/* need to decide V4 or V6 */
	if (memchr(ipaddr, '.', 4) != NULL)
		return simple_iplist_ipv4_add_string(head, ipaddr, data);

	return simple_iplist_ipv6_add_string(head, ipaddr, data);
}


/* Add a string in the format 192.168.1.0/24 to the iplist
 *
 * Data must be allocated
 */
static int simple_iplist_ipv4_add_string(struct simple_iphead* head, char * ipaddr, void *data)
{
	in_addr_t ip;
	in_addr_t subnet = 0xffffffff;
	int cidr = 32;
	char *tmpptr;
	char ipbuffer[INET_ADDRSTRLEN+1];

	if ( head == NULL ) {
		return IPLIST_ERR_NOMEM;
	}

	if ( ! simple_iplist_ipaddr_check(ipaddr) ) {
		return IPLIST_ERR_INVAL;
	}

	tmpptr = strchr(ipaddr, '/');
	if (tmpptr == NULL) {
		ip = inet_addr(ipaddr);
	}
	else {
		// the ip is to big, invalid, fail
		if ( tmpptr - ipaddr > INET_ADDRSTRLEN ) {
			return IPLIST_ERR_INVAL;
		}
		strncpy(ipbuffer, ipaddr, tmpptr-ipaddr);
		ipbuffer[tmpptr-ipaddr] = '\0';
		tmpptr++;

		ip = inet_addr(ipbuffer);
		cidr = atoi(tmpptr);
		if ( (ip != 0 && cidr == 0) || cidr < 0 || cidr > 32) {
			return IPLIST_ERR_INVAL;
		}
		subnet = htonl(subnet << ( 32 - cidr ));
		ip = ip & subnet;
	}

	// Do not add 255.255.255.255 (invalid from inet_addr function call)
	if ( ip == INADDR_NONE ) {
		return IPLIST_ERR_INVAL;
	}

	// ip 0.0.0.0 force cidr of zero
	if ( ip == 0 ) {
		cidr = 0;
	}


	// does this already exists
	if ( patricia_find_exact(head->head, ip, cidr) == 0) {
		return IPLIST_ERR_EXIST;
	}

	// add to the head
	if ( patricia_add(head->head, ip, cidr, data) ) {
		return IPLIST_ERR_TREE;
	}

	return IPLIST_SUCCESS;
}

// -----------------------
// IPv6 in strings fucntions
// -----------------------

/* Add a string in the format ipv6 to the iplist
 *
 * Data must be allocated
 */
static int simple_iplist_ipv6_add_string(struct simple_iphead* head, char * ipaddr, void *data)
{
	struct in6_addr ip;
	int cidr = 128;
	char *tmpptr;
	char ipbuffer[INET6_ADDRSTRLEN + 1];

	if ( head == NULL ) {
		return IPLIST_ERR_NOMEM;
	}

	tmpptr = strchr(ipaddr, '/');

	if (tmpptr == NULL) {
		if ( inet_pton(AF_INET6, ipaddr, &ip) <= 0) {
			return IPLIST_ERR_INVAL;
		}
	} else {
		// the ip is to big, invalid, fail
		if ( tmpptr - ipaddr > INET6_ADDRSTRLEN ) {
			return IPLIST_ERR_INVAL;
		}
		strncpy(ipbuffer, ipaddr, tmpptr-ipaddr);
		ipbuffer[tmpptr-ipaddr] = '\0';
		tmpptr++;

		if ( inet_pton(AF_INET6, ipbuffer, &ip) <= 0) {
			return IPLIST_ERR_INVAL;
		}

		cidr = atoi(tmpptr);
		if ( cidr < 0 || cidr > 128) {
			return IPLIST_ERR_INVAL;
		}

		simple_iplist_ipv6_mask_cidr(&ip, cidr);
	}

	// ip 0.0.0.0 force cidr of zero
	if ( ip.s6_addr32[0] == 0 && ip.s6_addr32[1] == 0 && ip.s6_addr32[2] == 0 && ip.s6_addr32[3] == 0) {
		cidr = 0;
	}

	// does this already exists
	if ( patricia_ipv6_find_exact(head->head_ipv6, &ip, cidr) == 0) {
		return IPLIST_ERR_EXIST;
	}

	// add to the head
	if ( patricia_ipv6_add(head->head_ipv6, &ip, cidr, data) ) {
		return IPLIST_ERR_TREE;
	}

	return IPLIST_SUCCESS;
}


/* Find an matching entry and return the private data
 */
void* simple_iplist_find_data(struct simple_iphead* head, ns_ipaddr *ip)
{
	if ( head == NULL ) {
		return NULL;
	}
	switch (ip->family) {
	case AF_INET:
		return patricia_find_data(head->head, ip->addr.ip);
	case AF_INET6:
		return patricia_ipv6_find_data(head->head_ipv6, &ip->addr.ip6);
	}
	return NULL;
}


static void simple_iplist_ipv6_mask_cidr(struct in6_addr *ip, int cidr)
{
	unsigned int mask = 0;

	switch (cidr) {
	case 0 ... 32 :
		ip->s6_addr32[1] = 0;
	case 33 ... 64 :
		ip->s6_addr32[2] = 0;
	case 65 ... 96 :
		ip->s6_addr32[3] = 0;
	}

	if (cidr < 32) {
		switch (cidr) {
		case 0 ... 8 :
			ip->s6_addr[1] = 0;
		case 9 ... 16 :
			ip->s6_addr[2] = 0;
		case 17 ... 24 :
			ip->s6_addr[3] = 0;
		}
	} else if (cidr < 64) {
		switch (cidr) {
		case 33 ... 40 :
			ip->s6_addr[5] = 0;
		case 41 ... 48 :
			ip->s6_addr[6] = 0;
		case 49 ... 56 :
			ip->s6_addr[7] = 0;
		}
	} else if (cidr < 96) {
		switch (cidr) {
		case 65 ... 72 :
			ip->s6_addr[9] = 0;
		case 73 ... 80 :
			ip->s6_addr[10] = 0;
		case 81 ... 88 :
			ip->s6_addr[11] = 0;
		}
	} else {
		switch (cidr) {
		case 97 ... 104 :
			ip->s6_addr[13] = 0;
		case 105 ... 112 :
			ip->s6_addr[14] = 0;
		case 113 ... 120 :
			ip->s6_addr[15] = 0;
		}
	}

	mask = 0xff << (8 - cidr % 8);
	ip->s6_addr[cidr/8] &= mask;
}


/* Initialize a new iplist head
 */
void* simple_iplist_init(void (*free_func)(void*), int (*match_func)(patricia_node_t* node))
{
	struct simple_iphead* head;
	head = malloc(sizeof(struct simple_iphead));
	if ( head == NULL ) {
		return NULL;
	}

	head->head = patricia_init(32, match_func);
	head->head_ipv6 = patricia_init(128, match_func);
	head->free_func = free_func;
	return head;
}


