/**
 * @file
 * @brief Tests to make sure the redirector can receive from squid and send request to NSP 
 * with different protocols properly
 *
 * Request can be sent from squid in different protocols
 * Request can be sent to NSP in different protocols
 *
 * Netsweeper Squid URL Redirector
 * This application is designed to work with the Netsweeper Internet Content Filtering System.
 *
 * Copyright 2010 Netsweeper Inc. All Rights Reserved.
 * Netsweeper, Netsweeper Inc. is either a copyrighted or registered trademark of Netsweeper Inc.
 */
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <pthread.h>
#include "jopen.h"

#include "CUnit/Basic.h"

#include "socketserver.h"

static char redir_nsp_buffer[256];

static void connect_callback(struct client_socket* s)
{
	printf("%p: Connected\n", s);
}
static void recv_callback(struct client_socket* s)
{
	int len;
	printf("%p: Data %s", s, s->rxbuffer);

	sprintf(redir_nsp_buffer, "%s", s->rxbuffer);

	if ( strchr(s->rxbuffer, '\n') ) {
		len = 0;
		while ( s->rxlen - len > 0 ) {
			socketserver_send(s, s->rxbuffer+len, 1);
			len++;
		}
		s->rxlen=0;
	}

}
static void disconnect_callback(struct client_socket* s)
{
	printf("%p: Disconnect\n", s);
}

void valid_squid25_output_test()
{
	struct socketserver* svr;
	JFILE* fp;
	int i;
	char sendbuffer[256];
	char buffer[256];
	char expbuffer[256];
	char expbuffer_nsp[256];
	int ret;

	// Create a new amazing server
	svr = socketserver_create(3435, connect_callback, recv_callback, disconnect_callback);

	// squid 2.5 input
	fp = jopen("../redir localhost -i squid2.5 -o squid2.5 -f http://failcloseurl");
	if ( ! fp ) {
		fprintf(stderr, "Could not open ../redir\n");
		socketserver_signal_shutdown(svr);
		socketserver_waitfor_shutdown(svr);
		return;
	}

	for(i=0; i<10; i++) {
		// simulate the buffer sent from squid
		sprintf(sendbuffer, "http://google.ca/%d -/- - -\n", i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - -\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "http://google.ca/%d -/- - -\n", i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.5 to squid2.5");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.5 to squid2.5");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	jclose(fp);
	CU_PASS("We have passed this test - squid2.5 to squid2.5");
	
	// squid 2.6 input
	fp = jopen("../redir localhost -i squid2.6 -o squid2.5 -f http://failcloseurl");
	if ( ! fp ) {
		fprintf(stderr, "Could not open ../redir\n");
		socketserver_signal_shutdown(svr);
		socketserver_waitfor_shutdown(svr);
		return;
	}

	for(i=0; i<10; i++) {
		sprintf(sendbuffer, "http://google.ca/%d -/- - - -\n", i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - -\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "http://google.ca/%d -/- - -\n", i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.5 to squid2.5");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.6 to squid2.5");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	CU_PASS("We have passed this test - squid2.6 to squid2.5");

	// concurrent squid26 input	
	for(i=0; i<10; i++) {
		sprintf(sendbuffer, "%d http://google.ca/%d -/- - - -\n", i+1, i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - -\n", i);

		// the buffer redir expects to send to squid
		sprintf(expbuffer, "%d http://google.ca/%d -/- - -\n", i+1, i);

		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - concurrent squid2.6 to squid2.5");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - concurrent squid2.6 to squid2.5");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	jclose(fp);
	CU_PASS("We have passed this test - squid2.6 to squid2.5");
	
	// squid2.7 input
	fp = jopen("../redir localhost -i squid2.7 -o squid2.5 -f http://failcloseurl");
	if ( ! fp ) {
		fprintf(stderr, "Could not open ../redir\n");
		socketserver_signal_shutdown(svr);
		socketserver_waitfor_shutdown(svr);
		return;
	}

	for(i=0; i<10; i++) {
		sprintf(sendbuffer, "http://google.ca/%d -/- - - - k1=v1 k2=v2\n", i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - -\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "http://google.ca/%d -/- - -\n", i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.5 to squid2.5");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != ecpect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.7 to squid2.5");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	CU_PASS("We have passed this test - squid2.7 to squid2.5");

	// concurrent squid27 input
	for(i=0; i<10; i++) {
		sprintf(sendbuffer, "%d http://google.ca/%d -/- - - - k1=v1 k2=v2\n", i+1, i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - -\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "%d http://google.ca/%d -/- - -\n", i+1, i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.5 to squid2.5");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - concurrent squid2.7 to squid2.5");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	jclose(fp);
	CU_PASS("We have passed this test - concurrent squid2.7 to squid2.5");

	// squid3.0 input
	fp = jopen("../redir localhost -i squid3.0 -o squid2.5 -f http://failcloseurl");
	if ( ! fp ) {
		fprintf(stderr, "Could not open ../redir\n");
		socketserver_signal_shutdown(svr);
		socketserver_waitfor_shutdown(svr);
		return;
	}

	for(i=0; i<10; i++) {
		sprintf(sendbuffer, "http://google.ca/%d -/- - - k1=v1 k2=v2\n", i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - -\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "http://google.ca/%d -/- - -\n", i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.5 to squid2.5");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - squid3.0 to squid2.5");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	CU_PASS("We have passed this test - squid3.0 to squid2.5");

	// concurrent squid30 input
	for(i=0; i<10; i++) {
		sprintf(sendbuffer, "%d http://google.ca/%d -/- - - k1=v1 k2=v2\n", i+1, i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - -\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "%d http://google.ca/%d -/- - -\n", i + 1, i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.5 to squid2.5");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - concurrent squid3.0 to squid2.5");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	jclose(fp);
	CU_PASS("We have passed this test - concurrent squid3.0 to squid2.5");
	
	socketserver_signal_shutdown(svr);
	socketserver_waitfor_shutdown(svr);
	return;
}

void valid_squid26_output_test()
{
	struct socketserver* svr;
	JFILE* fp;
	int i;
	char sendbuffer[256];
	char buffer[1024];
	char expbuffer[256];
	char expbuffer_nsp[256];
	int ret;

	// Create a new amazing server
	svr = socketserver_create(3429, connect_callback, recv_callback, disconnect_callback);

	// squid 2.5 input
	fp = jopen("../redir localhost:3429 -i squid2.5 -o squid2.6 -f http://failcloseurl");
	if ( ! fp ) {
		fprintf(stderr, "Could not open ../redir\n");
		socketserver_signal_shutdown(svr);
		socketserver_waitfor_shutdown(svr);
		return;
	}

	for(i=0; i<10; i++) {
		// simulate the buffer sent from squid
		sprintf(sendbuffer, "http://google.ca/%d -/- - -\n", i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - - -\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "http://google.ca/%d -/- - - -\n", i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL && len < 1023) {
			ret=read(fp->from_stdout, buffer+len, 1023-len);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.5 to squid2.6");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.5 to squid2.6");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	jclose(fp);
	CU_PASS("We have passed this test - squid2.5 to squid2.6");
	
	// squid 2.6 input
	fp = jopen("../redir localhost:3429 -i squid2.6 -o squid2.6 -f http://failcloseurl");
	if ( ! fp ) {
		fprintf(stderr, "Could not open ../redir\n");
		socketserver_signal_shutdown(svr);
		socketserver_waitfor_shutdown(svr);
		return;
	}

	for(i=0; i<10; i++) {
		sprintf(sendbuffer, "http://google.ca/%d -/- - - -\n", i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - - -\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "http://google.ca/%d -/- - - -\n", i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.6 to squid2.6");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.6 to squid2.6");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	CU_PASS("We have passed this test - squid2.6 to squid2.6");
	
	// squid 2.6 concurrent input
	for(i=0; i<10; i++) {
		sprintf(sendbuffer, "%d http://google.ca/%d -/- - - -\n", i+1, i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - - -\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "%d http://google.ca/%d -/- - - -\n", i+1, i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.6 to squid2.6 concurrent");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.6 to squid2.6 concurrent");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	jclose(fp);
	CU_PASS("We have passed this test - squid2.6 to squid2.6 concurrent");
	
	// squid2.7 input
	fp = jopen("../redir localhost:3429 -i squid2.7 -o squid2.6 -f http://failcloseurl");
	if ( ! fp ) {
		fprintf(stderr, "Could not open ../redir\n");
		socketserver_signal_shutdown(svr);
		socketserver_waitfor_shutdown(svr);
		return;
	}

	for(i=0; i<10; i++) {
		sprintf(sendbuffer, "http://google.ca/%d -/- - - - k1=v1 k2=v2\n", i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - - -\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "http://google.ca/%d -/- - - -\n", i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.7 to squid2.6");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.7 to squid2.6");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	CU_PASS("We have passed this test - squid2.7 to squid2.6");

	// squid2.7 concurrent input
	for(i=0; i<10; i++) {
		sprintf(sendbuffer, "%d http://google.ca/%d -/- - - - k1=v1 k2=v2\n", i+1, i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - - -\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "%d http://google.ca/%d -/- - - -\n", i+1, i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.7 to squid2.6 concurrent");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.7 to squid2.6 concurrent");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	jclose(fp);
	CU_PASS("We have passed this test - squid2.7 to squid2.6 concurrent");

	// squid3.0 input
	fp = jopen("../redir localhost:3429 -i squid3.0 -o squid2.6 -f http://failcloseurl");
	if ( ! fp ) {
		fprintf(stderr, "Could not open ../redir\n");
		socketserver_signal_shutdown(svr);
		socketserver_waitfor_shutdown(svr);
		return;
	}

	for(i=0; i<10; i++) {
		sprintf(sendbuffer, "http://google.ca/%d -/- - - k1=v1 k2=v2\n", i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - - -\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "http://google.ca/%d -/- - - -\n", i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid3.0 to squid2.6");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - squid3.0 to squid2.6");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	CU_PASS("We have passed this test - squid3.0 to squid2.6");
	
	// squid3.0 concurrent input
	for(i=0; i<10; i++) {
		sprintf(sendbuffer, "%d http://google.ca/%d -/- - - k1=v1 k2=v2\n", i+1, i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - - -\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "%d http://google.ca/%d -/- - - -\n", i+1, i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid3.0 to squid2.6 concurrent");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - squid3.0 to squid2.6 concurrent");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	jclose(fp);
	CU_PASS("We have passed this test - squid3.0 to squid2.6 concurrent");
	
	socketserver_signal_shutdown(svr);
	socketserver_waitfor_shutdown(svr);
	return;
}

void valid_squid30_output_test()
{
	struct socketserver* svr;
	JFILE* fp;
	int i;
	char sendbuffer[256];
	char buffer[256];
	char expbuffer[256];
	char expbuffer_nsp[256];
	int ret;

	// Create a new amazing server
	svr = socketserver_create(3428, connect_callback, recv_callback, disconnect_callback);

	// squid 2.5 input
	fp = jopen("../redir localhost:3428 -i squid2.5 -o squid3.0 -f http://failcloseurl");
	if ( ! fp ) {
		fprintf(stderr, "Could not open ../redir\n");
		socketserver_signal_shutdown(svr);
		socketserver_waitfor_shutdown(svr);
		return;
	}

	for(i=0; i<10; i++) {
		// simulate the buffer sent from squid
		sprintf(sendbuffer, "http://google.ca/%d -/- - -\n", i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - -\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "http://google.ca/%d -/- - -\n", i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.5 to squid3.0");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.5 to squid3.0");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	jclose(fp);
	CU_PASS("We have passed this test - squid2.5 to squid3.0");
	
	// squid 2.6 input
	fp = jopen("../redir localhost:3428 -i squid2.6 -o squid2.6 -f http://failcloseurl");
	if ( ! fp ) {
		fprintf(stderr, "Could not open ../redir\n");
		socketserver_signal_shutdown(svr);
		socketserver_waitfor_shutdown(svr);
		return;
	}

	for(i=0; i<10; i++) {
		sprintf(sendbuffer, "http://google.ca/%d -/- - - -\n", i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - -\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "http://google.ca/%d -/- - -\n", i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.6 to squid3.0");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.6 to squid3.0");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	CU_PASS("We have passed this test - squid2.6 to squid3.0");
	
	// squid 2.6 concurrent input
	for(i=0; i<10; i++) {
		sprintf(sendbuffer, "%d http://google.ca/%d -/- - - -\n", i+1, i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - -\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "%d http://google.ca/%d -/- - -\n", i+1, i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.6 to squid3.0 concurrent");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.6 to squid3.0 concurrent");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	jclose(fp);
	CU_PASS("We have passed this test - squid2.6 to squid2.6 concurrent");
	
	// squid2.7 input
	fp = jopen("../redir localhost:3428 -i squid2.7 -o squid2.6 -f http://failcloseurl");
	if ( ! fp ) {
		fprintf(stderr, "Could not open ../redir\n");
		socketserver_signal_shutdown(svr);
		socketserver_waitfor_shutdown(svr);
		return;
	}

	for(i=0; i<10; i++) {
		sprintf(sendbuffer, "http://google.ca/%d -/- - - - k1=v1 k2=v2\n", i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - - k1=v1 k2=v2\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "http://google.ca/%d -/- - - k1=v1 k2=v2\n", i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.7 to squid3.0");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.7 to squid3.0");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	CU_PASS("We have passed this test - squid2.7 to squid3.0");

	// squid2.7 concurrent input
	for(i=0; i<10; i++) {
		sprintf(sendbuffer, "%d http://google.ca/%d -/- - - - k1=v1 k2=v2\n", i+1, i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - - k1=v1 k2=v2\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "%d http://google.ca/%d -/- - - k1=v1 k2=v2\n", i+1, i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.7 to squid3.0 concurrent");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - squid2.7 to squid3.0 concurrent");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	jclose(fp);
	CU_PASS("We have passed this test - squid2.7 to squid3.0 concurrent");

	// squid3.0 input
	fp = jopen("../redir localhost:3428 -i squid3.0 -o squid3.0 -f http://failcloseurl");
	if ( ! fp ) {
		fprintf(stderr, "Could not open ../redir\n");
		socketserver_signal_shutdown(svr);
		socketserver_waitfor_shutdown(svr);
		return;
	}

	for(i=0; i<10; i++) {
		sprintf(sendbuffer, "http://google.ca/%d -/- - - k1=v1 k2=v2\n", i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - - k1=v1 k2=v2\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "http://google.ca/%d -/- - - k1=v1 k2=v2\n", i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid3.0 to squid3.0");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - squid3.0 to squid3.0");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	CU_PASS("We have passed this test - squid3.0 to squid3.0");
	
	// squid3.0 concurrent input
	for(i=0; i<10; i++) {
		sprintf(sendbuffer, "%d http://google.ca/%d -/- - - k1=v1 k2=v2\n", i+1, i);

		// the buffer redir expects to send to nsp
		sprintf(expbuffer_nsp, "http://google.ca/%d -/- - - k1=v1 k2=v2\n", i);

		// the buffer redir expects to send
		sprintf(expbuffer, "%d http://google.ca/%d -/- - - k1=v1 k2=v2\n", i+1, i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		if ( strcmp(redir_nsp_buffer, expbuffer_nsp) ) {
			fprintf(stderr, "real buffer sent to nsp %s != expect buffer %s\n", redir_nsp_buffer, expbuffer_nsp);
			CU_FAIL("We did not get the redirector output we expected for test - squid3.0 to squid3.0 concurrent");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}

		if ( strcmp(buffer, expbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != expect buffer %s\n", ret, buffer, expbuffer);
			CU_FAIL("We did not get the redirector output we expected for test - squid3.0 to squid3.0 concurrent");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	jclose(fp);
	CU_PASS("We have passed this test - squid3.0 to squid3.0 concurrent");
	
	socketserver_signal_shutdown(svr);
	socketserver_waitfor_shutdown(svr);
	return;
}

int redir_input_output_test_init()
{
	CU_pSuite pSuite = NULL;

	pSuite = CU_add_suite("Redir input and output Tests", NULL, NULL);
	if (NULL == pSuite) {
		CU_cleanup_registry();
		return CU_get_error();
	}

	if ( (NULL == CU_add_test(pSuite, "Valid Input to Squid25 Output\n", valid_squid25_output_test)) ) {
		CU_cleanup_registry();
		return CU_get_error();
	}

	if ( (NULL == CU_add_test(pSuite, "Valid Input to Squid26 Output\n", valid_squid26_output_test)) ) {
		CU_cleanup_registry();
		return CU_get_error();
	}

	if ( (NULL == CU_add_test(pSuite, "Valid Input to Squid30 Output\n", valid_squid26_output_test)) ) {
		CU_cleanup_registry();
		return CU_get_error();
	}

	return 0;
}
