/**
 * @file
 * @brief A simple select based socket server for unit tests
 *
 *
 *
 * Netsweeper Squid URL Redirector
 * This application is designed to work with the Netsweeper Internet Content Filtering System.
 *
 * Copyright 2010 Netsweeper Inc. All Rights Reserved.
 * Netsweeper, Netsweeper Inc. is either a copyrighted or registered trademark of Netsweeper Inc.
 */

#include <pthread.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>

#include "socketserver.h"

/** @brief The timeout in seconds for a connection */
#define CLIENT_TIMEOUT_SECONDS 60

/** @brief Timeout for a select, we do maintenance every timeout */
#define SELECT_TIMEOUT_SECONDS 1

static void* socketserver_loop(struct socketserver* srv);

/**
 * @brief initialize a socket for usage
 * @param cs The socket we want to init
 * @param srv The server this is a socket for
 */
static void socketserver_client_socket_init(struct client_socket* cs, struct socketserver* srv)
{
	cs->rxlen=0;
	cs->txlen=0;
	cs->remove=0;
	cs->socket=-1;
	cs->tag = NULL;
	cs->srv = srv;
}

/**
 * @brief Create a new server reference in a thread
 *
 * @param connect_callback Function to call on connect
 * @param recv_callback Function to call when we get data
 * @param disconnect_callback Function to call when we disconnect
 * @return struct socketserver* The socket server we created, NULL on error
 */
struct socketserver* socketserver_create(int port, SOCKETSERVER_CALLBACK connect_callback, SOCKETSERVER_CALLBACK recv_callback, SOCKETSERVER_CALLBACK disconnect_callback)
{
	struct socketserver* svr = malloc(sizeof(struct socketserver));

	svr->running = 1;
	svr->reset = 0;
	svr->port = port;
	svr->connect_callback = connect_callback;
	svr->recv_callback = recv_callback;
	svr->disconnect_callback = disconnect_callback;
	svr->tag = NULL;
	svr->tagfree_func = NULL;

	pthread_rwlock_init(&svr->start_lock, NULL);
	pthread_rwlock_rdlock(&svr->start_lock);

	pthread_create(&svr->threadid, NULL, (void* (*)(void*))socketserver_loop, svr);
	pthread_rwlock_wrlock(&svr->start_lock);
	pthread_rwlock_unlock(&svr->start_lock);
	return svr;
}
/**
 * @brief Set a tag free for the server
 *
 * @param svr The server we need to set a tag free for
 * @param func The function we call to free the server custom tag data
 */
void socketserver_tagfree_func(struct socketserver* svr, TAGFREE_CALLBACK func)
{
	svr->tagfree_func = func;
}

/**
 * @brief Our main server loop for processing events
 *
 * @param svr The server we are processing events for
 * @return void* NULL always and forever
 */
static void* socketserver_loop(struct socketserver* svr)
{
	int server_sockfd;
	struct client_socket* client;
	int client_sockfd;
	socklen_t server_len;
	socklen_t client_len;
	struct sockaddr_in server_address;
	struct sockaddr_in client_address;
	int result;
	fd_set readfds, testfds;
	int nread;
	int fd;
	struct timeval tv;
	int tryagain=20;
	int ret;
	int i;

	client = malloc(sizeof(struct client_socket)*FD_SETSIZE);

	// setup our socket
	for (i=0; i< tryagain; i++) {
		server_sockfd = socket(AF_INET, SOCK_STREAM, 0);
		
		// we want to be able to reuse the addr/port
		result = 1;
		setsockopt(server_sockfd, SOL_SOCKET , SO_REUSEADDR, &result, sizeof(int));
		
		server_address.sin_family = AF_INET;
		server_address.sin_addr.s_addr = htonl(INADDR_ANY);
		server_address.sin_port = htons(svr->port);
		server_len = (socklen_t) sizeof(server_address);
		if ( (ret=bind(server_sockfd, (struct sockaddr *)&server_address, server_len)) != 0) {
			char cmd[128];
			sprintf(cmd, "/bin/netstat -na | /bin/grep %d", svr->port );
			system(cmd);
			system("/sbin/fuser 12345/tcp");
			system("/bin/ps xal");
			close(server_sockfd);
			sleep(1);
			continue;
		}
		break;
	}
	if ( ret != 0 ) {
		perror("socketserver: ");
		shutdown(server_sockfd, SHUT_RDWR);
		close(server_sockfd);
		pthread_rwlock_unlock(&svr->start_lock);
		return NULL;
	}
	listen(server_sockfd, 10);

	FD_ZERO(&readfds);
	FD_SET(server_sockfd, &readfds);

	for(fd = 0; fd < FD_SETSIZE; fd++)
	{
		socketserver_client_socket_init(&client[fd], svr);
	}

	// We are running, unlock the start lock
	pthread_rwlock_unlock(&svr->start_lock);

	// start waiting for connections
	while( svr->running ) {
		// timeout every second to see if the server is going down
		tv.tv_sec = SELECT_TIMEOUT_SECONDS;
		tv.tv_usec = 0;
		testfds = readfds;

		result = select(FD_SETSIZE, &testfds, (fd_set *)0, 
			(fd_set *)0, &tv);

		// <0 means termination, 0 is a timeout
		if(result < 0)
			break;

		// if all connecteion reset requested flag all connectuons for close
		if ( svr->reset ) {
			for(fd = 0; fd < FD_SETSIZE; fd++) {
				if ( client[fd].socket > 0 && fd != server_sockfd ) {
					client[fd].remove = 1;

				}
			}
		}

		// loop through all connections
		for(fd = 0; fd < FD_SETSIZE; fd++)
		{
			// this connection has data
			if(FD_ISSET(fd,&testfds))
			{
				// server connection == accept client
				if(fd == server_sockfd)
				{
					// accept client connection
					client_len = (socklen_t) sizeof(client_address);
					client_sockfd = accept(server_sockfd,
							(struct sockaddr *)&client_address, &client_len);

					socketserver_client_socket_init(&client[fd], svr);

					client[client_sockfd].socket = client_sockfd;

					// set the last action time
					gettimeofday(&tv, NULL);
					client[client_sockfd].last_action_time = tv.tv_sec;

					if ( svr->connect_callback ) {
						svr->connect_callback(&client[client_sockfd]);
					}

					FD_SET(client_sockfd, &readfds);
				}

				// client data connection == accept data
				else {
					ioctl(fd, FIONREAD, &nread);
					// disconnect
					if ( nread <= 0 )
						client[fd].remove = 1;
					else
					{

						nread=read(fd, client[fd].rxbuffer+client[fd].rxlen, CLIENT_BUFFER_SIZE-client[fd].rxlen - 1);

						// make sure we read something
						if ( nread > 0 ) {
							client[fd].rxlen += nread;
							client[fd].rxbuffer[client[fd].rxlen] = '\0';
							if ( svr->recv_callback ) {
								svr->recv_callback(&client[fd]);
							}
						}

						gettimeofday(&tv, NULL);
						client[fd].last_action_time = tv.tv_sec;
					}

				}
			}
			// fd was not set, check if it timed out
			else if ( client[fd].socket > 0 )
			{
				gettimeofday(&tv, NULL);
				if ( client[fd].last_action_time < tv.tv_sec - CLIENT_TIMEOUT_SECONDS ) {
					client[fd].remove = 1;
				}
			}

			// check if the client should be removed
			if ( client[fd].remove ) {
				if ( svr->disconnect_callback ) {
					svr->disconnect_callback(&client[fd]);
				}

				shutdown(fd, SHUT_RDWR);
				close(fd);
				FD_CLR(fd, &readfds);

				client[fd].socket = -1;
				client[fd].remove = 0;
			}

		}
		svr->reset = 0;
	}

	// disconnect any open sessions on server termination
	nread=0;
	for(fd = 0; fd < FD_SETSIZE; fd++)
	{
		if ( client[fd].socket > 0 )
		{
			nread++;

			// close the connection
			shutdown(client[fd].socket, SHUT_RDWR);
			close(client[fd].socket);

			if ( svr->disconnect_callback ) {
				svr->disconnect_callback(&client[fd]);
			}
		}
	}

	if ( server_sockfd != -1 ) {
		shutdown(server_sockfd, SHUT_RDWR);
		close(server_sockfd);
		server_sockfd=-1;
	}
	free(client);

	return NULL;
}

/**
 * @brief Send some data on a socket.  We continue to send until we give up
 *
 * Do not use this in a production server.  This is just an example in a
 * very simple framework.  Will fail on large data segments.
 *
 * @param s The socket we are sending on
 * @param buffer The buffer we are using to send
 * @param len the length of the buffer we are sending
 *
 * @return int the number of bytes send
 */
int socketserver_send(struct client_socket* s, char* buffer, int len)
{
	char *line;
	int retval;
	int bytes_left;

	// send the data to the squid client
	bytes_left = len;
	line = buffer;
	while( bytes_left > 0 ) {
		retval = write(s->socket, line, bytes_left);

		if ( retval <= 0 ) {
			perror("socketserver send error:");
			return -1;
		}

		bytes_left -= retval;
		line+=retval;
	}

	return len;
}

/**
 * @brief Flag this socket we should disconnect it
 *
 * @param s The socket we wish to shutdown
 * @return int 0 always
 */
int socketserver_connection_shutdown(struct client_socket* s)
{
	s->remove=1;
	return 0;
}

/**
 * @brief Signal the server event loop we want to shutdown
 *
 * @param svr The server we wish to shutdown
 */
void socketserver_signal_shutdown(struct socketserver* svr)
{
	svr->running = 0;
}

/**
 * @brief Function used to wait for a server to properly terminate
 *
 * This will signal as well, so you can just call this function directly.
 *
 * @param svr The server we wish to shutdown
 */
void socketserver_waitfor_shutdown(struct socketserver* svr)
{
	socketserver_signal_shutdown(svr);

	pthread_join(svr->threadid, NULL);
	if ( svr->tagfree_func ) {
		svr->tagfree_func(svr->tag);
	}
	pthread_rwlock_destroy(&svr->start_lock);
	free(svr);
}

/**
 * @brief Signal the server to close all active connections but don't exit
 *
 * @param svr The server which connections we wish to reset
 */
void socketserver_signal_reset_connections(struct socketserver* svr)
{
	svr->reset = 1;
}

/**
 * @brief Function used to wait for a server close all connections
 *
 * @param svr The server which connections we wait
 */
void socketserver_waitfor_reset_connections(struct socketserver* svr)
{
	// a simplest spin lock
	while (svr->reset == 1) {
		usleep(1);
	}
}
